unit Unit1;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Types3D,
  System.Math.Vectors, FMX.Objects3D, FMX.Controls3D, Gorilla.Camera,
  Gorilla.Light, Gorilla.Viewport, Gorilla.AssetsManager, Gorilla.Control,
  Gorilla.Mesh, Gorilla.Model, Gorilla.Utils.Path, FMX.Ani, Gorilla.Bokeh,
  Gorilla.Particle.Influencer, Gorilla.Particle.Emitter, FMX.MaterialSources,
  Gorilla.Material.Default, Gorilla.Material.Particle, Gorilla.Material.Lambert,
  Gorilla.Sphere, Gorilla.Transform, Gorilla.Cube, Gorilla.SkyBox;

type
  TForm1 = class(TForm)
    GorillaViewport1: TGorillaViewport;
    GorillaLight1: TGorillaLight;
    GorillaCamera1: TGorillaCamera;
    Dummy1: TDummy;
    ReefModel: TGorillaModel;
    GorillaAssetsManager1: TGorillaAssetsManager;
    Timer1: TTimer;
    FishModel: TGorillaModel;
    GorillaPath3DAnimation1: TGorillaPath3DAnimation;
    GorillaParticleEmitter1: TGorillaParticleEmitter;
    GorillaParticleMaterialSource1: TGorillaParticleMaterialSource;
    GorillaBokeh1: TGorillaBokeh;
    GorillaSphere1: TGorillaSphere;
    GorillaLambertMaterialSource1: TGorillaLambertMaterialSource;
    GorillaSkyBox1: TGorillaSkyBox;
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}

uses
  System.IOUtils,
  Gorilla.DefTypes,
  Gorilla.FBX.Loader,
  Gorilla.DAE.Loader;

procedure TForm1.FormCreate(Sender: TObject);
var LPath : String;
    LFile : String;
    LPckg : TGorillaAssetsPackage;
    LPrst : TParticleVectorPreset;
begin
  GorillaBokeh1.DepthPass.Enabled := false;
  GorillaBokeh1.MainPass.Enabled := false;

  // get the package created at designtime
  LPckg := GorillaAssetsManager1.GetPackage('InMemory');
  if not Assigned(LPckg) then
    Exit;

  // setup assets path
{$IFDEF MSWINDOWS}
  LPath := IncludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0)));
{$ELSE}
  LPath := IncludeTrailingPathDelimiter(System.IOUtils.TPath.GetHomePath());
{$ENDIF}
  LPath := LPath + IncludeTrailingPathDelimiter('assets');

  // load reef
  LFile := LPath + IncludeTrailingPathDelimiter('reef') + 'reef-mod.dae';
  ReefModel.LoadFromFile(LPckg, LFile, []);

  // load animated fish model
  LFile := LPath + IncludeTrailingPathDelimiter('fish') + 'fish.fbx';
  FishModel.LoadFromFile(LPckg, LFile, GORILLA_ANIMATION_CACHING_DEFAULT);
  FishModel.Position.Point := Point3D(0, -5, 0);
  FishModel.Scale.Point := Point3D(0.04, 0.04, 0.04);
  FishModel.RotationAngle.X := 90;

  // create a 3D path to move the fish
  with GorillaPath3DAnimation1.Path do
  begin
    MoveTo(Point3D(0, -5, -3));
    LineTo(Point3D(0, -5, 5));
    LineTo(Point3D(-5, -5, 5));
    LineTo(Point3D(-5, -5, -3));
    LineTo(Point3D(0, -5, -3));
    ClosePath();
  end;

  // create bubble particle effect
  LPrst := TParticleVectorPreset.Create(0, 50, 10, Point3D(1, -1, 1), [axX, axZ]);
  GorillaParticleEmitter1.ParticlePosition := LPrst;

  LPrst := TParticleVectorPreset.Create(1, 10, 10, Point3D(0, -1, 0));
  LPrst.RangeY.Negative := false;
  GorillaParticleEmitter1.ParticleVelocity := LPrst;
  GorillaParticleEmitter1.ParticleSize     := TParticlePreset.Create(4, 4, 1, false);
  GorillaParticleEmitter1.ParticleGrowth   := TParticlePreset.Create(1, 1, 1, false);
  GorillaParticleEmitter1.ParticleLifeTime := TParticlePreset.Create(2, 10, 1, false);

  // start bubbles
  GorillaParticleEmitter1.Start;

  // start moving fish
  FishModel.AnimationManager.Current.Start();
  GorillaPath3DAnimation1.Start();

  // activate auto camera rotation
  Timer1.Enabled := true;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  GorillaViewport1.BeginUpdate();
  try
    Dummy1.RotationAngle.Y := Dummy1.RotationAngle.Y + 0.25;
  finally
    GorillaViewport1.EndUpdate();
  end;

  Self.Caption := Format('Gorilla3D - Aquarium Demo [%n FPS]',
    [GorillaViewport1.FPS]);
end;

end.
